from __future__ import annotations

import streamlit as st
from config import to_session_state
from image_sim import image_similarity_feat_ransac_widget
from styling import set_custom_css
from widgets import (
    load_config_widget,
    load_images_widget,
    show_heatmaps_widget,
    show_images_widget,
)


def main():
    set_custom_css()

    with st.sidebar:
        load_config_widget()
        st.write('---')
        images = load_images_widget()

    st.title('Input images')

    _ = show_images_widget(images)

    st.title('Image similarity')

    col, _ = st.columns(2)

    method_options = ['sift', 'orb', 'outline', 'scale']
    method = col.selectbox(
        'Select similarity metric',
        options=method_options,
        key='method',
        on_change=to_session_state,
        kwargs={'key': 'method'},
    )

    if method in ('sift', 'orb'):
        heatmaps, features = image_similarity_feat_ransac_widget(images, method=method)
    else:
        raise NotImplementedError(method)
        st.stop()

    show_heatmaps_widget(heatmaps=heatmaps, labels=tuple(features.keys()))


if __name__ == '__main__':
    main()



from __future__ import annotations

from pathlib import Path

import cv2
import numpy as np
import platformdirs
import streamlit as st
from styling import set_custom_css
from ultralytics import YOLO
from widgets import image_downloads_widget, load_image_widget

from unraphael.pose import BodyDrawer

CACHEDIR = Path(platformdirs.user_cache_dir('unraphael'))

MODELS = {
    'Detection': CACHEDIR / 'yolov8n.pt',
    'Segmentation': CACHEDIR / 'yolov8x-seg.pt',
    'Pose': CACHEDIR / 'yolov8x-pose.pt',
}


@st.cache_resource
def load_model(model_type: str):
    model_path = MODELS[model_type]
    try:
        model = YOLO(model_path)
    except Exception as ex:
        st.error(f'Unable to load model. Check the specified path: {model_path}')
        st.error(ex)
    return model


def _detection_task(*, results):
    objects = {}

    for result in results:
        for ci, c in enumerate(result):
            image = result.orig_img

            label = c.names[c.boxes.cls.tolist().pop()]
            x1, y1, x2, y2 = c.boxes.xyxy.cpu().numpy().squeeze().astype(int)
            # Crop the object from the image
            new_image = image[y1:y2, x1:x2]

            objects[f'{label}_{ci}'] = new_image

    return objects


def _segmentation_task(*, results):
    objects = {}

    for result in results:
        for ci, c in enumerate(result):
            image = result.orig_img

            label = c.names[c.boxes.cls.tolist().pop()]
            b_mask = np.zeros(image.shape[:2], np.uint8)
            contour = c.masks.xy.pop().astype(int).reshape(-1, 1, 2)
            cv2.drawContours(b_mask, [contour], -1, (255, 255, 255), cv2.FILLED)
            new_image = np.dstack([image, b_mask])

            objects[f'{label}_{ci}'] = new_image

    return objects


def _pose_task(*, results):
    objects = {}

    for idx, result in enumerate(results):
        for ci, keypoints in enumerate(result.keypoints.data):
            img = result.orig_img

            frame_height, frame_width = img.shape[:2]

            new_image = np.zeros((frame_height, frame_width, 3), dtype=np.uint8)
            keypoints = keypoints.cpu().numpy()
            label = f'pose_{idx}'

            BodyDrawer(keypoints).draw(image=new_image)

            objects[f'{label}_{ci}'] = new_image

    return objects


def yolo_task_widget(image: np.ndarray, /) -> dict[str, np.ndarray]:
    """This widget takes an image and provides the user with some choices of which tasks
    to perform: Detection, Segmentation, Pose analysis."""
    col1, col2, col3 = st.columns([0.30, 0.35, 0.35])

    task_name = col1.radio('Select Task', [None, 'Detection', 'Segmentation', 'Pose'])
    add_box = col1.checkbox('Add bounding box', value=True)
    confidence = float(col1.slider('Select Model Confidence', 10, 100, 25)) / 100

    col2.image(image, caption='Uploaded Image', use_column_width=True)

    if not task_name:
        st.info('Select a task to continue')
        st.stop()

    model = load_model(task_name)

    results = model.predict(
        image,
        conf=confidence,
        show_boxes=True,
        save=True,
        save_txt=True,
    )

    if len(results[0].boxes) == 0:
        st.error('No objects detected in the image.')

    result_image = results[0].plot(boxes=add_box)

    col3.image(result_image, caption='Detected Image', use_column_width=True)

    if task_name == 'Detection':
        images = _detection_task(results=results)
    elif task_name == 'Segmentation':
        images = _segmentation_task(results=results)
    elif task_name == 'Pose':
        images = _pose_task(results=results)

    return images


def main():
    set_custom_css()

    st.title('Segmentation of figures in a painting')

    name, image = load_image_widget()

    images = yolo_task_widget(image)

    image_downloads_widget(images=images)


if __name__ == '__main__':
    main()



from __future__ import annotations

import base64

import streamlit as st

from unraphael.locations import data_directory


def set_custom_css(stylesheet: str = 'custom.css'):
    """Set custom css here."""
    add_sidebar_logo()

    fn = str(data_directory / stylesheet)

    with open(fn) as f:
        data = f.read()

    st.write(
        f'<style>\n{data}\n</style>',
        unsafe_allow_html=True,
    )


@st.cache_data
def get_base64_of_bin_file(png_file):
    with open(png_file, 'rb') as f:
        data = f.read()
    return base64.b64encode(data).decode()


def build_markup_for_logo(
    png_file,
    background_position='50% 10%',
    margin_top='10%',
    image_width='60%',
    image_height='',
    nav=True,  # page has navigation
):
    binary_string = get_base64_of_bin_file(png_file)

    loc = 'stSidebarHeader' if nav else 'stSidebarUserContent'

    return f"""
            <style>
                [data-testid="{loc}"] {{
                    background-image: url("data:image/png;base64,{binary_string}");
                    background-repeat: no-repeat;
                    background-position: {background_position};
                    margin-top: {margin_top};
                    background-size: {image_width} {image_height};
                }}
            </style>
            """


def add_sidebar_logo():
    """Based on: https://stackoverflow.com/a/73278825."""
    png_file = data_directory / 'logo-dark.png'
    logo_markup = build_markup_for_logo(png_file, nav=True)
    st.markdown(
        logo_markup,
        unsafe_allow_html=True,
    )



from __future__ import annotations

import streamlit as st
from config import dump_config, to_session_state

from unraphael.feature import (
    detect_and_extract,
    get_heatmaps,
)

_detect_and_extract = st.cache_data(detect_and_extract)
_get_heatmaps = st.cache_data(get_heatmaps)


def image_similarity_feat_ransac_widget(images, *, method: str):
    """Image similarity for ORB/SIFT."""
    col1, col2 = st.columns(2)

    col1.subheader('Feature extraction')
    col2.subheader('Image matching')

    if method == 'sift':
        label1 = '[SIFT config parameters](https://scikit-image.org/docs/stable/api/skimage.feature.html#skimage.feature.SIFT)'
        st.session_state.col1 = dump_config(st.session_state.config['sift'])
    elif method == 'orb':
        label1 = '[ORB config parameters](https://scikit-image.org/docs/stable/api/skimage.feature.html#skimage.feature.ORB)'
        st.session_state.col1 = dump_config(st.session_state.config['orb'])
    else:
        raise ValueError(method)

    col1.text_area(
        label=label1,
        key='col1',
        height=200,
        on_change=to_session_state,
        kwargs={'key': 'col1', 'section': method},
    )

    label2 = '[RANSAC config parameters](https://scikit-image.org/docs/stable/api/skimage.measure.html#skimage.measure.ransac)'
    st.session_state.col2 = dump_config(st.session_state.config['ransac'])

    col2.text_area(
        label=label2,
        key='col2',
        height=200,
        on_change=to_session_state,
        kwargs={'key': 'col2', 'section': 'ransac'},
    )

    if not st.checkbox('Continue...', key='continue_ransac'):
        st.stop()

    features = _detect_and_extract(
        images=images, method=method, **st.session_state.config[method]
    )

    heatmaps = _get_heatmaps(features, **st.session_state.config['ransac'])

    return heatmaps, features



from __future__ import annotations

from dataclasses import dataclass
from typing import Any

import matplotlib.pyplot as plt
import numpy as np
from skimage.feature import ORB, SIFT, match_descriptors
from skimage.measure import ransac
from skimage.transform import FundamentalMatrixTransform


@dataclass
class FeatureContainer:
    name: str
    image: np.array
    descriptors: np.array | None = None
    keypoints: np.array | None = None
    scales: np.array | None = None

    def plot_keypoints(self):
        fig, ax = plt.subplots()

        ax.imshow(self.image, cmap=plt.cm.gray)
        ax.scatter(
            self.keypoints[:, 1],
            self.keypoints[:, 0],
            2**self.scales,
            facecolors='none',
            edgecolors='r',
        )
        ax.set_title(self.name)
        return fig


def detect_and_extract(
    images: dict[str, np.ndarray], *, method: str, **kwargs
) -> dict[str, FeatureContainer]:
    """`extractor` must have `detect_and_extract` method."""
    features = {}

    if method == 'sift':
        extractor = SIFT(**kwargs)
    elif method == 'orb':
        extractor = ORB(**kwargs)
    else:
        raise ValueError(method)

    for i, (name, im) in enumerate(images.items()):
        extractor.detect_and_extract(im)

        features[name] = FeatureContainer(
            name=name,
            image=im,
            keypoints=extractor.keypoints,
            descriptors=extractor.descriptors,
            scales=extractor.scales,
        )

    return features


def get_heatmaps(
    features: dict[str, FeatureContainer],
    progress: Any = None,
    min_samples=8,
    residual_threshold=1,
    max_trials=5000,
    **kwargs,
) -> dict[str, np.array]:
    n = len(features)

    heatmap = np.zeros((n, n), dtype=int)
    heatmap_inliers = np.zeros((n, n), dtype=int)

    features_tup = tuple(features.values())

    for (i1, i2), _ in np.ndenumerate(heatmap):
        if i1 >= i2:
            continue

        ft1 = features_tup[i1]
        ft2 = features_tup[i2]

        matches = match_descriptors(ft1.descriptors, ft2.descriptors, cross_check=True)

        heatmap[i1, i2] = heatmap[i2, i1] = len(matches)

        try:
            model, inliers = ransac(
                (ft1.keypoints[matches[:, 0]], ft2.keypoints[matches[:, 1]]),
                FundamentalMatrixTransform,
                min_samples=min_samples,
                residual_threshold=residual_threshold,
                max_trials=max_trials,
                **kwargs,
            )

        except ValueError:
            inliers = np.zeros(len(matches), dtype=bool)

        heatmap_inliers[i1, i2] = heatmap_inliers[i2, i1] = inliers.sum()

    return {'all': heatmap, 'inliers': heatmap_inliers}


def heatmap_to_condensed_distance_matrix(heatmap: np.ndarray) -> np.ndarray:
    heatmap = heatmap / heatmap.max()
    dmat = np.sqrt(1 - heatmap**2)

    # array must be a condensed distance matrix
    tri = np.triu_indices_from(dmat, k=1)
    d = dmat[tri]
    return d



from __future__ import annotations

from importlib.resources import files

dash_directory = files('unraphael.dash')
data_directory = files('unraphael.data')
image_directory = data_directory / 'images'



from __future__ import annotations

from typing import TYPE_CHECKING

import imageio.v3 as imageio
import streamlit as st
from config import (
    _load_config,
    _update_session_state,
    dump_session_state,
    to_session_state,
)
from scipy.cluster.hierarchy import linkage
from seaborn import clustermap

from unraphael.feature import (
    heatmap_to_condensed_distance_matrix,
)
from unraphael.io import load_images, load_images_from_drc
from unraphael.locations import data_directory, image_directory

if TYPE_CHECKING:
    import numpy as np


_load_images = st.cache_data(load_images)
_load_images_from_drc = st.cache_data(load_images_from_drc)


def show_images_widget(images: dict[str, np.ndarray], *, n_cols: int = 4):
    """Widget to show images with given number of columns."""
    col1, col2 = st.columns(2)
    n_cols = col1.number_input('Number of columns', value=8, min_value=1, step=1)
    selected = col2.selectbox('Selected', options=images.keys())

    cols = st.columns(n_cols)

    for i, (name, im) in enumerate(images.items()):
        if i % n_cols == 0:
            cols = st.columns(n_cols)
        col = cols[i % n_cols]
        col.image(im, use_column_width=True, caption=name)

    return selected


def load_image_widget() -> tuple[str, np.ndarray]:
    """Widget to load a single image with default."""
    load_example = st.sidebar.checkbox('Load example', value=False, key='load_example')
    uploaded_file = st.sidebar.file_uploader('Upload Image ', type=['JPG', 'JPEG'])

    if load_example:
        image_file = image_directory / '0_edinburgh_nat_gallery.jpg'
    else:
        if not uploaded_file:
            st.info('Upload image to continue')
            st.stop()

        image_file = uploaded_file

    name, _ = image_file.name.rsplit('.')
    image = imageio.imread(image_file)

    return name, image


def load_config_widget():
    """Widget to load config file."""
    uploaded_file = st.sidebar.file_uploader('Load config ', type=['yml', 'yaml'])
    if not uploaded_file:
        uploaded_file = data_directory / 'config.yaml'

    config = _load_config(uploaded_file)
    if 'config' not in st.session_state:
        _update_session_state(config)

    st.download_button(
        label='Download config',
        data=dump_session_state(),
        file_name='my_config.yaml',
        mime='text/yaml',
        disabled=False,
    )


def load_images_widget():
    """Widget to load images."""

    load_example = st.sidebar.checkbox('Load example', value=False, key='load_example')
    uploaded_files = st.file_uploader('Upload Images', accept_multiple_files=True)

    width = st.number_input(
        'Width',
        step=10,
        key='width',
        on_change=to_session_state,
        kwargs={'key': 'width'},
    )

    if load_example:
        images = _load_images_from_drc(image_directory, width=width)
    else:
        if not uploaded_files:
            st.info('Upload images to continue')
            st.stop()

        images = _load_images(uploaded_files, width=width)

    if not images:
        raise ValueError('No images were loaded')

    return images


def show_heatmaps_widget(heatmaps: dict[str, np.ndarray], labels: list[str]):
    """Widget to show heatmaps."""
    st.title('Heatmaps')

    col, _ = st.columns(2)

    options = 'single', 'average', 'complete', 'median', 'weighted', 'centroid', 'ward'
    method = col.selectbox('Linking method', options=options, index=1)

    cols = st.columns(len(heatmaps))

    for col, (name, heatmap) in zip(cols, heatmaps.items()):
        col.subheader(name.capitalize())

        d = heatmap_to_condensed_distance_matrix(heatmap)
        z = linkage(d, method=method)

        fig = clustermap(
            heatmap,
            xticklabels=labels,
            yticklabels=labels,
            annot=True,
            fmt='d',
            row_linkage=z,
            col_linkage=z,
        )

        col.pyplot(fig)


def image_downloads_widget(*, images: dict[str, np.ndarray], basename: str | None = None):
    """This widget takes a dict of images and shows them with download
    buttons."""
    st.title('Download Images')

    prefix = f'{basename}_' if basename else ''

    cols = st.columns(len(images))

    for col, key in zip(cols, images):
        image = images[key]

        col.image(image, caption=key.upper(), use_column_width=True)

        filename = f'{prefix}{key}.png'

        col.download_button(
            label=f'Download ({filename})',
            data=imageio.imwrite('<bytes>', image, extension='.png'),
            file_name=filename,
            key=filename,
        )



from __future__ import annotations

import cv2
import numpy as np
import rembg


def apply_mask(original_image: np.ndarray, mask: np.ndarray) -> np.ndarray:
    """Apply mask to original image."""
    return cv2.bitwise_and(original_image, original_image, mask=mask)


def process_image(
    image: np.array,
    *,
    bilateral_strength: int,
    clahe_clip_limit: float,
    clahe_tiles: int,
    sigma_sharpness: float,
    contrast: float,
    brightness: int,
    sharpening_kernel_size: int,
    saturation_factor: float,
) -> np.ndarray:
    """Process the uploaded image with user-defined parameters.

    Parameters
    ----------
    image : np.array
        Input image
    bilateral_strength : int
        Apply bilateral blur filter to each color channel
    clahe_clip_limit : float
        Clip limit for histogram equalization
    clahe_tiles : int
        Number of tiles for histogram equalization
    contrast : float
      Change contrast
    brightness : int
        Change brightness
    sigma_sharpness : float
    sharpening_kernel_size : int
        Sharpen image with this kernel size
    saturation_factor : float
        Multiply the saturation channel by user-defined factor

    Returns
    -------
    image : np.ndarray
        Processed image
    """
    # Check if the image is grayscale and convert it to 3 channels if necessary
    if len(image.shape) == 2:
        image = cv2.cvtColor(image, cv2.COLOR_GRAY2BGR)

    # Split PIL image into its individual color channels
    blue, green, red = cv2.split(image)

    # Apply bilateral blur filter to each color channel with user-defined 'bilateral_strength'
    blue_blur = cv2.bilateralFilter(blue, d=bilateral_strength, sigmaColor=55, sigmaSpace=55)
    green_blur = cv2.bilateralFilter(green, d=bilateral_strength, sigmaColor=55, sigmaSpace=55)
    red_blur = cv2.bilateralFilter(red, d=bilateral_strength, sigmaColor=55, sigmaSpace=55)

    # Create CLAHE object with user-defined clip limit
    clahe = cv2.createCLAHE(clipLimit=clahe_clip_limit, tileGridSize=(clahe_tiles, clahe_tiles))

    # Adjust histogram and contrast for each color channel using CLAHE
    blue_eq = clahe.apply(blue_blur)
    green_eq = clahe.apply(green_blur)
    red_eq = clahe.apply(red_blur)

    # Merge the color channels back into a single RGB image
    output_img = cv2.merge((blue_eq, green_eq, red_eq))

    # Color saturation: convert image from BGR color space to HSV (Hue, Saturation, Value)
    # color space
    hsv_image = cv2.cvtColor(output_img, cv2.COLOR_BGR2HSV)

    # Multiply the saturation channel by user-defined 'saturation_factor'
    hsv_image[:, :, 1] = np.clip(hsv_image[:, :, 1] * saturation_factor, 1, 254).astype(
        np.uint8
    )

    # Convert image back to BGR color space
    result_image = cv2.cvtColor(hsv_image, cv2.COLOR_HSV2BGR)

    # Create user-defined 'sharpening_kernel_size'
    kernel = np.ones((sharpening_kernel_size, sharpening_kernel_size), np.float32) * -1
    kernel[sharpening_kernel_size // 2, sharpening_kernel_size // 2] = sharpening_kernel_size**2

    # Apply sharpening kernel to image using filter2D
    processed_image = cv2.filter2D(result_image, -1, kernel)

    # Alpha controls contrast and beta controls brightness
    processed_image = cv2.convertScaleAbs(processed_image, alpha=contrast, beta=brightness)

    # Additional sharpening: Create the sharpening kernel and apply it to the image
    custom_kernel = np.array([[-1, -1, -1], [-1, 9, -1], [-1, -1, -1]])

    # Sharpen the image
    processed_image = cv2.filter2D(processed_image, -1, custom_kernel)

    # Apply Gaussian blur to the image with user-defined 'sigma_sharpness'
    processed_image = cv2.GaussianBlur(processed_image, (0, 0), sigma_sharpness)

    return processed_image


def remove_background(image: np.ndarray, mask_process: bool = False, **kwargs) -> np.ndarray:
    """Remove background using rembg.

    Parameters
    ----------
    image : np.ndarray
        Input image
    mask_process : bool
        Set `only_mask` and `post_process_mask` to True
    **kwargs
        These parameters are passed to rembg.remove

    Returns
    -------
    image : np.ndarray
        Image with background removed
    """
    if mask_process:
        kwargs['only_mask'] = True
        kwargs['post_process_mask'] = True

    image = rembg.remove(image, **kwargs)

    return image



from __future__ import annotations

from dataclasses import dataclass

import cv2
import numpy as np


@dataclass
class BodyDrawer:
    keypoints: np.ndarray
    HEAD: tuple[int, int, int] = (0, 255, 0)  # green
    BODY: tuple[int, int, int] = (0, 0, 255)  # blue
    LEGS: tuple[int, int, int] = (255, 165, 0)  # orange
    REST: tuple[int, int, int] = (128, 0, 128)  # purple

    def draw(self, *, image):
        self.draw_points_head(image=image)
        self.draw_points_legs(image=image)
        self.draw_points_body(image=image)
        self.draw_skeleton_legs(image=image)
        self.draw_skeleton_body(image=image)
        self.draw_skeleton_head(image=image)
        self.draw_additional(image=image)

    def _draw_points(self, *, image, indices, color):
        for j, kp in enumerate(self.keypoints):
            if j in indices:
                if kp[2] > 0.5:
                    x, y = (
                        int(kp[0]),
                        int(kp[1]),
                    )
                    cv2.circle(image, (x, y), 5, color, -1)

    def draw_points_head(self, *, image):
        # indices for eyes, ears, and nose
        head_indices = [0, 1, 2, 3, 4]
        self._draw_points(image=image, indices=head_indices, color=self.HEAD)

    def draw_points_body(self, *, image):
        # indices for shoulders, elbows, and wrists
        body_indices = [5, 6, 7, 8, 9, 10]
        self._draw_points(image=image, indices=body_indices, color=self.BODY)

    def draw_points_legs(self, *, image):
        # indices for hips, knees, and ankles
        leg_indices = [11, 13, 15, 12, 14, 16]
        self._draw_points(image=image, indices=leg_indices, color=self.LEGS)

    def _draw_skeleton(self, *, image, indices, color):
        for j1, j2 in indices:
            kp1 = self.keypoints[j1]
            kp2 = self.keypoints[j2]

            # check confidence of both keypoints
            if kp1[2] > 0.5 and kp2[2] > 0.5:
                p1 = (int(kp1[0]), int(kp1[1]))
                p2 = (int(kp2[0]), int(kp2[1]))
                cv2.line(image, p1, p2, color, 2)

    def draw_skeleton_legs(self, *, image):
        # left hip to left knee, left knee to left ankle
        # right hip to right knee, right knee to right ankle
        indices = [(11, 13), (13, 15), (12, 14), (14, 16)]
        self._draw_skeleton(image=image, indices=indices, color=self.LEGS)

    def draw_skeleton_body(self, *, image):
        # shoulder to shoulder, shoulder to elbow, elbow to wrist
        indices = [(5, 6), (5, 7), (7, 9), (6, 8), (8, 10)]
        self._draw_skeleton(image=image, indices=indices, color=self.BODY)

    def draw_skeleton_head(self, *, image):
        # nose to eyes, eyes to ears
        indices = [(0, 1), (1, 2), (0, 3), (3, 4), (0, 2), (2, 4)]
        self._draw_skeleton(image=image, indices=indices, color=self.HEAD)

    def draw_additional(self, *, image):
        # left ear to left shoulder, right ear to right shoulder
        self._draw_skeleton(image=image, indices=[(3, 5)], color=self.HEAD)
        # right ear to right shoulder, left ear to left shoulder
        self._draw_skeleton(image=image, indices=[(4, 6)], color=self.HEAD)
        # left shoulder to left hip
        self._draw_skeleton(image=image, indices=[(5, 11)], color=self.REST)
        # right shoulder to right hip
        self._draw_skeleton(image=image, indices=[(6, 12)], color=self.REST)
        # left shoulder to right shoulder
        self._draw_skeleton(image=image, indices=[(5, 6)], color=self.REST)
        # left hip to right hip
        self._draw_skeleton(image=image, indices=[(11, 12)], color=self.REST)



from __future__ import annotations

from collections.abc import Sequence
from pathlib import Path
from typing import TYPE_CHECKING, Any

import imageio.v3 as imageio
from skimage.color import rgb2gray
from skimage.transform import resize

if TYPE_CHECKING:
    import numpy as np


def load_images(
    image_files: Sequence[Any], *, width: int, as_gray: bool = True
) -> dict[str, np.ndarray]:
    """Load images through `imageio.imread` and do some preprocessing."""
    images = {}

    for image_file in image_files:
        im = imageio.imread(image_file)
        name, _ = image_file.name.rsplit('.')

        if as_gray and im.ndim > 2:
            im = rgb2gray(im)

        x, y = im.shape
        k = y / width
        new_shape = int(x / k), int(y / k)
        im = resize(im, new_shape)

        images[name] = im

    return images


def load_images_from_drc(drc: Path, **kwargs) -> dict[str, np.array]:
    """Load all images in directory.

    kwargs are passed to `load_images`.
    """
    fns = list(drc.glob('*'))

    return load_images(fns, **kwargs)



from __future__ import annotations

import logging
import sys

import toml

from .locations import dash_directory

logger = logging.getLogger(__name__)


def dash_entry(**kwargs):
    """Start streamlit dashboard."""
    from streamlit.web import cli as stcli

    argv = sys.argv[1:]

    dashboard_path = dash_directory / 'home.py'

    config_file = dash_directory / '.streamlit' / 'config.toml'
    config = toml.load(config_file)

    opts = []
    for group, options in config.items():
        for option, value in options.items():
            opts.extend((f'--{group}.{option}', str(value)))

    sys.argv = ['streamlit', 'run', str(dashboard_path), *opts, *argv]

    print(sys.argv)

    logger.debug('Streamlit arguments %s', sys.argv)

    sys.exit(stcli.main())



from __future__ import annotations

import numpy as np
import streamlit as st
from styling import set_custom_css
from widgets import image_downloads_widget, load_image_widget

from unraphael.preprocess import apply_mask, process_image, remove_background

_process_image = st.cache_data(process_image)
_apply_mask = st.cache_data(process_image)
_remove_background = st.cache_data(remove_background)


def preprocess_image_widget(image: np.ndarray):
    """Widget to preprocess image with user input options."""
    st.title('Preprocessing')
    st.write(
        'The processed image is shown with a preset of parameters. '
        'Use the sliders to explore the effects of image filters, or to'
        'refine the adjustment. When you are happy with the result, '
        'download the processed image.'
    )

    col1, col2, col3, col4 = st.columns(4)

    image_params = {}

    image_params['bilateral_strength'] = col1.slider(
        'Bilateral Filter Strength',
        min_value=0,
        max_value=15,
        value=5,
        key='bilateral',
        help='(default = 5)',
    )
    image_params['saturation_factor'] = col1.slider(
        'Color Saturation',
        min_value=0.0,
        max_value=2.0,
        step=0.05,
        value=1.1,
        key='saturation',
        help='(default = 1.1)',
    )
    image_params['clahe_clip_limit'] = col1.slider(
        'CLAHE Clip Limit',
        min_value=0.0,
        max_value=5.0,
        value=2.0,
        step=0.05,
        key='clahe',
        help='Threshold for contrast limiting (default = 2)',
    )
    image_params['clahe_tiles'] = col1.slider(
        'CLAHE Tile Grid Size',
        min_value=2,
        max_value=15,
        value=8,
        step=1,
        key='tiles',
        help='Tile size for local contrast enhancement (default = 8)',
    )

    image_params['sigma_sharpness'] = col2.slider(
        'Sharpness Sigma',
        min_value=0.0,
        max_value=3.0,
        value=0.5,
        step=0.1,
        key='sharpness',
        help='(default = 0.5)',
    )
    image_params['contrast'] = col2.slider(
        'Contrast',
        min_value=0.1,
        max_value=3.0,
        value=1.0,
        step=0.1,
        key='contrast',
        help='(default = 1.0)',
    )
    image_params['brightness'] = col2.slider(
        'Brightness',
        min_value=-100,
        max_value=100,
        value=10,
        step=1,
        key='brightness',
        help='(default = 10)',
    )
    image_params['sharpening_kernel_size'] = col2.slider(
        'Sharpening kernel size',
        min_value=1,
        max_value=9,
        step=2,
        value=3,
        key='sharpen',
        help='(default = 3)',
    )
    out = process_image(image, **image_params)
    col3.image(image, 'before')
    col4.image(out, 'after')
    return out


def remove_background_widget(image: np.ndarray) -> np.ndarray:
    """This widget takes an image and provides the user with some choices to
    remove the background."""
    st.title('Background removal')
    st.write('Change these parameters to tune the background removal.')

    col1, col2, col3, col4 = st.columns(4)

    background_params = {}

    background_params['alpha_matting'] = col1.checkbox(
        'Use Alpha matting',
        value=False,
        help=(
            'Alpha matting is a post processing step that can be used to '
            'improve the quality of the output.'
        ),
    )
    background_params['only_mask'] = col1.checkbox('Keep mask only', value=False)
    background_params['post_process_mask'] = col1.checkbox('Postprocess mask', value=False)

    bgmap = {
        (0, 0, 0, 0): 'Transparent',
        (255, 255, 255, 255): 'White',
        (0, 0, 0, 255): 'Black',
    }

    background_params['bgcolor'] = col1.radio(
        'Background color',
        bgmap.keys(),
        format_func=lambda x: bgmap[x],
        help=(
            'You can use the post_process_mask argument to post process the '
            'mask to get better results.'
        ),
    )

    background_params['bg_threshold'] = col2.slider(
        'Background threshold',
        min_value=0,
        max_value=255,
        value=10,
        key='background',
        help='(default = 10)',
    )
    background_params['fg_threshold'] = col2.slider(
        'Foreground threshold',
        min_value=0,
        max_value=255,
        value=200,
        key='foreground',
        help='(default = 200)',
    )
    background_params['erode_size'] = col2.slider(
        'Erode size',
        min_value=0,
        max_value=25,
        value=10,
        key='erode',
        help='(default = 10)',
    )

    nobg = remove_background(image, **background_params, mask_process=False)

    mask = remove_background(image, **background_params, mask_process=True)

    col3.image(mask, 'mask')
    col4.image(nobg, 'background removed')

    return nobg, mask


def main():
    set_custom_css()

    name, image = load_image_widget()

    processed = preprocess_image_widget(image)
    processed_nobg, processed_mask = remove_background_widget(processed)

    images = {
        'original': image,
        'processed': processed_nobg,
        'extracted': apply_mask(image, processed_mask),
    }

    image_downloads_widget(basename=name, images=images)


if __name__ == '__main__':
    main()



from __future__ import annotations

import streamlit as st
from styling import data_directory, set_custom_css

from unraphael import __version__

st.set_page_config(
    page_title='Unraphael dashboard',
    page_icon='🖼️',
    layout='centered',
    initial_sidebar_state='auto',
    menu_items={
        'Get Help': 'https://unraphael.readthedocs.io',
        'Report a bug': 'https://github.com/DedodingRaphael/unraphael/issues',
        'About': (
            f'**unraphael**: a dashboard for unraphael ({__version__}). '
            '\n\nPython toolkit for *unraveling* image similarity with a focus '
            'on artistic practice. '
            '\n\nFor more information, see: https://github.com/DedodingRaphael/unraphael'
        ),
    },
)

set_custom_css()

st.image(str(data_directory / 'logo-dark.png'))

st.markdown(
    """
Unraphael is a digital workflow tool that uses computer vision to unravel the artistic
practice of Raphael (Raffaello Sanzio, 1483-1520), while providing new digital approaches
for the study of artistic practice in art history. Dozens of faithful reproductions survive
of Raphael's paintings, attesting to the lucrative practice of serial production of
paintings within the artist's workshop and to the lasting demand for the master's designs.

This tool aims to provide new insights into Raphael's working methods through new digital
approaches for the study of artistic practice in art history.

### Pages

- <a href="/image_similarity" target="_parent">Image Similarity</a>
- <a href="/preprocess" target="_parent">Preprocess</a>
- <a href="/detect_objects" target="_parent">Detect objects</a>

### More information

- [Source code](https://github.com/DecodingRaphael/unraphael)
- [Documentation](https://unraphael.readthedocs.io/)
""",
    unsafe_allow_html=True,
)



from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING

import streamlit as st
import yaml

if TYPE_CHECKING:
    from io import IOBase


def to_session_state(key: str, section: str | None = None):
    section = section if section else key

    try:
        st.session_state.config[section] = yaml.safe_load(st.session_state[key])
    except AttributeError:
        st.session_state.config[section] = st.session_state[key]
    except KeyError:
        print(f'Could not commit {key}->{section} in config')


def dump_session_state():
    return dump_config(st.session_state.config)


def dump_config(cfg: dict) -> str:
    def cfg_str_representer(dumper, in_str):
        if '\n' in in_str:  # use '|' style for multiline strings
            return dumper.represent_scalar('tag:yaml.org,2002:str', in_str, style='|')
        return dumper.represent_scalar('tag:yaml.org,2002:str', in_str)

    yaml.representer.SafeRepresenter.add_representer(str, cfg_str_representer)
    return yaml.safe_dump(cfg, sort_keys=False, allow_unicode=True)


def _update_session_state(config: dict):
    st.session_state.config = config
    st.session_state.width = config['width']
    st.session_state.method = config['method']


@st.cache_data
def _load_config(fn: Path | str | IOBase) -> dict:
    """Load config into session state and return copy."""
    if isinstance(fn, (Path, str)):
        with open(fn) as f:
            config = yaml.safe_load(f)
    else:
        config = yaml.safe_load(fn)

    _update_session_state(config)

    return config
